#include "z3-api/c++/z3++.h"
#include <vector>
#include <string>
#include <iostream>
#include <cmath>

void make_state(z3::context& ctx, int num, z3::expr_vector& res)
{
    res.resize(0);
    res.push_back(ctx.bool_const(("v_0^" + std::to_string(num)).c_str()));
    res.push_back(ctx.bool_const(("v_1^" + std::to_string(num)).c_str()));
}

void make_inital(z3::solver& solver, const z3::expr_vector& state)
{
    solver.add(!state[0]);
    solver.add(!state[1]);
}

void make_trans(z3::solver& solver, const z3::expr_vector& curr, const z3::expr_vector& next)
{
    z3::expr trans = solver.ctx().bool_val(false);
    trans = trans || (!curr[0] && !curr[1] && next[0] && !next[1]);
    trans = trans || (curr[0] && !curr[1] && next[1]);
    trans = trans || (!curr[0] && curr[1] && !next[0] && !next[1]);
    trans = trans || (curr[0] && curr[1] && next[0] && next[1]);
    solver.add(trans);
}

z3::expr make_a(const z3::expr_vector& state)
{
    return !state[0] || !state[1];
}

int main() {
    z3::context ctx;
    z3::solver solver(ctx);

    z3::expr_vector  s0(ctx);
    make_state(ctx, 0, s0);
    make_inital(solver, s0);
    z3::expr a0 = make_a(s0);

    z3::expr_vector  s1(ctx);
    make_state(ctx, 1, s1);
    make_trans(solver, s0, s1);
    z3::expr a1 = make_a(s1);

    z3::expr_vector  s2(ctx);
    make_state(ctx, 2, s2);
    make_trans(solver, s1, s2);
    z3::expr a2 = make_a(s2);

    solver.add(!a0 || !a1 || !a2);

    z3::check_result res = solver.check();
    if (res == z3::check_result::sat)
    {
        z3::model model = solver.get_model();
        std::cout << model;
    } else
    {
        std::cout << "impossible" << std::endl;
    }

    return 0;
}
